<?php

/**
 * Class api4marketingPro
 *
 * Classe di gestione dei metodi di 4Dem
 */
class api4marketingPro {

    private $server;
    private $apiKey;
    private $sessionID;
    private $status;
    private $response;

    /**
     * api4marketingPro constructor.
     * @param $server
     * @param $apiKey
     */
    public function __construct($server,$apiKey) {

        $this->server       = $server;
        $this->apiKey       = $apiKey;

        $this->status       = false;
        $this->token        = false;

        $this->login();
    }

    /**
     * Chiamata alle API 4Dem
     *
     * @param array $data
     * @param string $command
     * @param int $method
     * @param int $test
     * @return mixed
     */
    private function call($data = array(), $command = "",$method = 1,$test=0) {


        //$argString = implode('&',$data);

        // echo "<pre>";
        //var_dump($argString);
        $defaults = array(
            CURLOPT_HEADER => 0,
            CURLOPT_POST => $method == 1 || $method == 2 ? true : false,
            CURLOPT_HTTPHEADER =>array('Content-Type: application/json'),
            CURLOPT_URL => $this->server."/".$command.($this->token!=false ? "?token=".$this->token : ""),
            CURLOPT_RETURNTRANSFER => 1
        );
        if($method==1) {
            $defaults[CURLOPT_POSTFIELDS] = json_encode($data);
        }else if($method==2) {
            $defaults[CURLOPT_POSTFIELDS] = json_encode($data);
            $defaults[CURLOPT_HEADER] = true;
            $defaults[CURLOPT_CUSTOMREQUEST] = "PUT";
        }else if ($method == 3) {
            $defaults[CURLOPT_HEADER] = true;
            $defaults[CURLOPT_POSTFIELDS] = json_encode($data);
            $defaults[CURLOPT_CUSTOMREQUEST] = "DELETE";
        }

        $logCmd = array();
        $logCmd['cmd'] = $command;
        $logCmd['data'] = $data;

        Configuration::updateValue ( '4DEM_LAST_CMD_CALLED', json_encode($logCmd) );

        $ch = curl_init();
        curl_setopt_array($ch, $defaults);

        $response = curl_exec($ch);
        if($test==1) {
            echo "<pre>";
            var_dump($response);
            echo "</pre>";

        } else if($test==2) {
            echo "<pre>";
            var_dump($defaults);
            echo "</pre>";

        }
        $response = json_decode($response);


        return $response;
    }

    /**
     * Richiesta del token di autenticazione
     *
     * @return bool
     */
    public function login() {

        $login = array();
        $login[] =

        $command= "authenticate";
        $defaults = array(
            CURLOPT_POST => 1,
            CURLOPT_HEADER => 0,
            CURLOPT_URL => $this->server."/authenticate",
            CURLOPT_USERAGENT => '4marketing',
            CURLOPT_FRESH_CONNECT => 0,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_FORBID_REUSE => 0,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_POSTFIELDS => "APIKey=".$this->apiKey
        );
        $ch = curl_init();
        curl_setopt_array($ch, $defaults);

        $response = curl_exec($ch);

        $response = json_decode($response);
        $this->response = $response;

        if (isset($response) && isset($response->token)) {
            $this->status = true;
            $this->token = $response->token;
        }else{
            $this->status = false;
        }
        return $this->status;

    }

    /**
     * Ritorna le liste presenti su 4Dem
     *
     * @return bool
     */
    public function getLists() {

        $commands="recipients";
        $response = $this->call(array(),$commands,0);

        $this->response = $response;
        if ($response) {
            if (isset($response->data)) {
                return $response->data;
            }
        }
        return false;
    }

    /**
     * Ritorna l'elenco degli store presenti su 4Dem
     *
     * @return bool
     */
    public function getStores(){

        $commands="stores";
        $response = $this->call(array(),$commands,0);

        $this->response = $response;
        if ($response) {
            if (isset($response->data)) {
                return $response->data;
            }
        }
        return false;
    }

    /**
     * Creazione di uno store
     *
     * @param $listId
     * @param $currency
     * @param $nome
     * @param $shop_id
     * @param $shop_url
     * @param array $attributes
     * @return mixed
     */
    public function createStore($listId,$currency,$nome,$shop_id,$shop_url,$attributes=array()){
        $command="stores";
        $params = array();
        $params["name"] = $nome;
        $params["type"]= "Prestashop";
        $params["recipient_id"]= $listId;
        $params["currency"]= $currency;
        $params["shop_id"]= $shop_id;
        $params["base_url"]= $shop_url;
        $params["attributes"] = $attributes;
        $response = $this->call($params,$command);
        $this->response = $response;
        return $response;
    }

    /**
     * Restituisce gli attributi dello store
     *
     * @param $storeId
     * @return bool
     */
    public function getStoreAttribute($storeId){
        $command="stores/".$storeId."/attributes";

        $response = $this->call(array(),$command,0);
        $this->response = $response;
        if ($response) {
            if (isset($response->data)) {
                return $response->data;
            }
        }
        return false;
    }

    /**
     * Invio massivo di dati a 4Dem
     *
     * @param $callback
     * @param $operations
     * @return mixed
     */
    public function massiveImport($callback,$operations) {
        $commands="batches";
        $args = array();
        $args["callback_url"] = $callback;
        $args["skip_error"] = true;
        $args["operations"] = $operations;
        $response = $this->call($args,$commands,1);
        $this->response = $response;

        return $response;


    }

    /**
     * Creazione del servizio web per la cancellazione dalla newsletter
     *
     * @param $callback
     * @return bool|mixed
     */
    public function createWebServiceUnsubscribe($callback) {
        $commands="web_services";
        $args = array();
        $args["recipient_id"] = Configuration::get('4DEM_SELECTED_LIST_ID');
        $args["type"] = "unsubscription";
        $args["url"] = $callback;
        $response = $this->call($args,$commands,1);
        $this->response = $response;

        if (is_numeric($response)) {
            return $response;
        }
        return false;
    }

    /**
     * Aggiornamento delle informazioni della lista
     *
     * @param $listId
     * @return bool
     */
    public function updList($listId){
        $command="recipients/".$listId;
        $params = array();
        $params["name"] = Context::getContext()->shop->domain." - ".Context::getContext()->shop->name." ID: ".Context::getContext()->shop->id;
        $integration = array();
        $integration["active"] = true;
        $integration["domain"] = Context::getContext()->shop->domain;
        $integration["shopid"] = Context::getContext()->shop->id;
        $integration["module"] = "prestashop";
        $integration["last_sync_date"] = (Configuration::get('4DEM_LAST_SYNC_CONTACTS') ? (Configuration::get('4DEM_LAST_SYNC_CONTACTS')) : 0);

        $params["integration"]=$integration;
        $response = $this->call($params,$command,2);
        $this->response = $response;
        if ($response) {
            return false;
        }
        return true;
    }

    /**
     * Archiviazione della lista
     *
     * @param $listId
     * @return bool
     */
    public function archiveList($listId){
        $command="recipients/".$listId;
        $params = array();
        $params["name"] = Context::getContext()->shop->domain." - ".Context::getContext()->shop->name." ID: ".Context::getContext()->shop->id." - Archived";
        $integration = array();
        $integration["active"] = false;
        $integration["domain"] = Context::getContext()->shop->domain;
        $integration["shopid"] = 0;
        $integration["module"] = "prestashop";
        $integration["last_sync_date"] = (Configuration::get('4DEM_LAST_SYNC_CONTACTS') ? (Configuration::get('4DEM_LAST_SYNC_CONTACTS')) : 0);

        $params["integration"]=$integration;
        $response = $this->call($params,$command,2);
        $this->response = $response;
        if ($response) {
            return false;
        }
        return true;
    }

    /**
     * Creazione della lista
     *
     * @return bool|string
     */
    public function createList() {
        $command="recipients";
        $params = array();
        $params["name"] = Context::getContext()->shop->domain." - ".Context::getContext()->shop->name." ID: ".Context::getContext()->shop->id;
        $params["opt_in"]["mode"] = "single";
        $params["opt_out"]["mode"] = "direct";
        $params["opt_out"]["add_to_global_suppression"] = false;
        $params["opt_out"]["add_to_local_suppression"] = false;

        $integration = array();
        $integration["active"] = true;
        $integration["domain"] = Context::getContext()->shop->domain;
        $integration["shopid"] = Context::getContext()->shop->id;
        $integration["module"] = "prestashop";
        // $integration["last_sync_date"] = date("Y-m-d h:i:s");

        $params["integration"]=$integration;
        Configuration::updateValue( '4DEM_CREATION_CMD', json_encode($params));
        $response = $this->call($params,$command);
        $this->response = $response;
        if (is_numeric($response)) {
            Configuration::updateValue( '4DEM_SELECTED_LIST_ID', $response);
            return true;
        }

        return (isset($response->status_code) ? $response->status_code.": " : "" ).json_encode($response->message);
    }

    /**
     * Aggiornamento della lista
     *
     * @param $listId
     * @return bool
     */
    public function updateList($listId) {
        $command="recipients/".$listId;
        $params = array();
        $params["name"] = "test";
        $params["opt_in"]["mode"] = "single";
        $params["opt_out"]["mode"] = "direct";
        $params["opt_out"]["add_to_global_suppression"] = false;
        $params["opt_out"]["add_to_local_suppression"] = false;
        $integration = array();
        $integration["active"] = true;
        $integration["domain"] = Context::getContext()->shop->domain;
        $integration["shopid"] = Context::getContext()->shop->id;
        $integration["module"] = "prestashop";
        $integration["last_sync_date"] = (Configuration::get('4DEM_LAST_SYNC_CONTACTS') ? (Configuration::get('4DEM_LAST_SYNC_CONTACTS')) : 0);

        $params["integration"]=$integration;
        Configuration::updateValue ( '4DEM_CMD_UPT', json_encode($params));

        $response = $this->call($params,$command,2);

        $this->response = $response;
        Configuration::updateValue ( '4DEM_CMD_UPT_RES', json_encode($response));
        if (is_null($response)) {
            return true;
        }

        return false;
    }

    /**
     * Verifica esistenza della lista
     *
     * @param $listId
     * @return bool
     */
    public function checkList($listId) {
        $command = "recipients/".$listId;

        $response = $this->call(array(),$command,0);

        $this->response = $response;

        if ($response) {
            if (isset($response->data)) {
                Configuration::updateValue( '4DEM_SELECTED_LIST_ID', $response->data->id);
                Configuration::updateValue( '4DEM_SELECTED_LIST_NAME', $response->data->name);
                return true;
            }
        }
        return false;
    }

    /**
     * Verifica congruenza dei campi personalizzati
     *
     * @param $listId
     * @param $fields
     * @return bool
     */
    public function checkCustomFields($listId, $fields) {

        // Campi su 4dem
        $DemFields = $this->getCustomFields($listId);
        if (is_null($DemFields)) {
            return false;
        } elseif ((sizeof($DemFields) < sizeof($fields))) {
            return 0;
        } else {
            foreach ($fields as $key => $value) {
                $found = false;
                foreach ($DemFields as $demField) {
                    if ($fields[$key]['name'] == $demField->name) {
                        $fields[$key]['id'] = $demField->id;
                        $found = true;
                    }
                }
                if (!$found) {
                    return false;
                }
            }
        }

        $json = array();
        foreach($fields as $key => $field){
            $json[$key]["ignore"] = false;
            $json[$key]["name"] = $field["name"];
            $json[$key]["id"] =$field["id"];

        }
        
        $json["cellphone"] = [];
        $json["cellphone"]["ignore"] = false;
        $json["cellphone"]["name"] = "N° Cellulare";


        Configuration::updateValue ( '4DEM_CUSTOM_FIELDS', json_encode($json));
        return true;
    }

    /**
     * Creazione campi personalizzati
     *
     * @param $listId
     * @param $fields
     * @param $callback
     * @return bool
     */
    public function createCustomFields($listId,$fields,$callback) {

        $command="batches";

        /*CREAZIONE SCHEMA CAMPI CUSTOM*/
        $json = array();
        $operations = array();
        foreach($fields as $key => $field){
            $customField = array();
            $customField["recipient_id"] = $listId;
            $customField["name"] = $field["name"];
            $customField["type"] = $field["type"];
            $customField["global"] = false;
            if($field["type"] == "Checkboxes"){
                $options = array();
                foreach($field["options"] as $key2 => $opt){
                    $options[$key2]["label"] = $opt["label"];
                    $options[$key2]["value"] = $opt["val"];
                    $options[$key2]["selected"] = false;

                }
                $customField["options"] = $options;
            }
            if($field["type"] == "Date field"){
                $customField["year"]["from"] = 1980;
                $customField["year"]["to"] = 2050;
            }
            $customField["default_value"] = $field["val"];
            $operations[] = array("command"=>"CustomField.Store","custom_id"=>$key,"body"=>$customField);

            //$response = $this->call($customField,$command);
            $json[$key]["ignore"] = false;
            $json[$key]["name"] = $field["name"];
            $json[$key]["id"] =false;

        }
        $batch = array("callback_url"=>$callback,"skip_error"=>false,"operations"=>$operations);

        Configuration::updateValue ( '4DEM_CREATE_FIELD_CMD', json_encode($batch) );
        $response = $this->call($batch,$command,1);
        Configuration::updateValue ( '4DEM_CREATE_FIELD', json_encode($response) );
       // Configuration::updateValue("4DEM_CUSTOM_FIELDS_WAITING",1);
        if(is_numeric($response)){
            $json["cellphone"] = [];
            $json["cellphone"]["ignore"] = false;
            $json["cellphone"]["name"] = "N° Cellulare";
            Configuration::updateValue ( '4DEM_CUSTOM_FIELDS', json_encode($json) );
            return true;
        }
        return false;
    }

    /**
     * Aggiornamento campi personalizzati
     *
     * @param $listId
     * @param $field
     * @param $fieldID
     * @return bool|string
     */
    public function updateCustomField($listId,$field,$fieldID) {
        /*CREAZIONE SCHEMA CAMPI CUSTOM*/
        $command = "custom_fields/".$fieldID;
        $args = array();
        $args["recipient_id"] = $listId;
        $args["name"] = $field["name"];
        $args["type"] = $field["type"];
        if($field["type"] == "Checkboxes"){
            $options = array();
            foreach($field["options"] as $key2 => $opt){
                $options[$key2]["label"] = $opt["label"];
                $options[$key2]["value"] = $opt["val"];
                $options[$key2]["selected"] = false;

            }
            $args["options"] = $options;
        }
        $response = $this->call($args,$command,2);
        if(!$response->Success) {
            $err = "Errore per ".$key2;
        }

        if ($err=="") {
            return true;
        }
        return $err;
    }

    /**
     * Ritorna i campi personalizzati
     *
     * @param $listId
     * @return bool
     */
    public function getCustomFields($listId) {

        $command = "recipients/".$listId."/custom_fields";

        $response = $this->call(array(),$command,0);

        $this->response = $response;
        if ($response) {
            if (isset($response->data)) {
                return $response->data;
            }
        }
        return false;
    }

    /**
     * Cancellazione della store
     *
     * @param $storeId
     * @return bool
     */
    public function deleteStore($storeId) {

        $command = "stores/".$storeId;

        $response = $this->call(array(),$command,3);
        Configuration::updateValue ( '4DEM_DELETE_CALL_RES', json_encode($response));
        $this->response = $response;
        if (is_null($response)) {
            return true;
        }
        return false;
    }

    /**
     * Registra gli utenti alla lista
     *
     * @param $listId
     * @param $email
     * @param string $mobile
     * @param bool $customFields
     * @param bool $customerInfo
     * @return bool|mixed
     */
    public function subscribe($listId,$email,$mobile = "",$customFields = false, $customerInfo = false) {

        $command="recipients/".$listId."/subscribe";

        $subscribe = array();
        $subscribe["update_if_duplicate"] = true;
        $subscribe["email_address"] = $email;
        if($mobile!="")
            $subscribe["mobile"] = urldecode($mobile);

        if ($customFields) {
            $subscribe["custom_fields"] = array();
            foreach ($customFields as $key => $field) {
                $subscribe["custom_fields"][$key]["id"] = $field;
                $subscribe["custom_fields"][$key]["value"] = $customerInfo[$key];
            }
        }
        $subscribe["subscription"]["ip"] = (isset($_SERVER['REMOTE_ADDR'])) ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0';

        $response = $this->call($subscribe,$command);
        $this->response = $response;

        if ($response && is_numeric($response)) {
            return $response;

        }
        return false;
    }

    /**
     * Ritorna le informazioni su uno specifico utente
     *
     * @param $listId
     * @param $id
     * @return bool
     */
    public function getSubscriber($listId,$id) {

        $command = "recipients/".$listId."/contacts/".$id;

        $response = $this->call(array(),$command,0);
        $this->response = $response;

        if ($response) {
            if (isset($response->data)) {
                return $response->data;
            }
        }
        return false;
    }

    /**
     * Aggiunge un carrello
     *
     * @param $storeId
     * @param $operations
     * @return bool|mixed
     */
    public function addCart($storeId,$operations){
        $command = "stores/".$storeId."/carts";

        $response = $this->call($operations,$command,1);
        $this->response = $response;
        return $response;
        if ($response) {
            if (isset($response->meta)) {
                return $response->meta->pagination->total;
            }
        }
        return false;

    }

    /**
     * Ritorna il numero di prodotti sullo store
     *
     * @param $storeId
     * @return bool
     */
    public function getProductsNumber($storeId) {
        $command = "stores/".$storeId."/products/all";

        $response = $this->call(array(),$command,0);
        $this->response = $response;
        if ($response) {
            if (isset($response->meta)) {
                return $response->meta->pagination->total;
            }
        }
        return false;
    }

    /**
     * Ritorna il numero di carrelli sullo store
     *
     * @param $storeId
     * @return bool
     */
    public function getCartsNumber($storeId) {
        $command = "stores/".$storeId."/carts";

        $response = $this->call(array(),$command,0);
        $this->response = $response;
        if ($response) {
            if (isset($response->meta)) {
                return $response->meta->pagination->total;
            }
        }
        return false;
    }

    /**
     * Ritorna il numero di iscritti alla newsletter
     *
     * @param $listId
     * @return bool
     */
    public function getSubscribersNumber($listId) {
        $command = "recipients/".$listId."/contacts";

        $response = $this->call(array(),$command,0);
        $this->response = $response;
        if ($response) {
            if (isset($response->meta)) {
                return $response->meta->pagination->total;
            }
        }
        return false;
    }

    /**
     * Cancella un utente dalla newsletter
     *
     * @param $listId
     * @param $id
     * @return bool
     */
    public function unsubscribe($listId,$id) {

        $command = "recipients/".$listId."/unsubscribe";
        $args = array();
        $args["subscriber_id"] = $id;
        $args["ip"] = (isset($_SERVER['REMOTE_ADDR'])) ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0';
        $response = $this->call($args,$command);
        $this->response = $response;
        if ($response) {
            if ($response->Success == true) {
                return true;
            }
        }
        return false;
    }

    /**
     * Ritorna la lista degli utenti cancellati dalla newsletter
     *
     * @param $listId
     * @return bool
     */
    public function getUnsubscribed($listId) {

        $subscriberGet = array();
        $subscriberGet[] = "SessionID=".$this->sessionID;
        $subscriberGet[] = "Command=Subscribers.Get";
        $subscriberGet[] = "ResponseFormat=JSON";
        $subscriberGet[] = "OrderField=UnsubscriptionDate";
        $subscriberGet[] = "OrderType=ASC";
        $subscriberGet[] = "RecordsFrom=0";
        $subscriberGet[] = "SubscriberListID=".$listId;
        $subscriberGet[] = "SubscriberSegment=Unsubscribed";

        $response = $this->call($subscriberGet);
        $this->response = $response;
        if ($response) {
            if ($response->Success == true) {
                return $response->Subscribers;
            }
        }
        return false;
    }

    /**
     * Ritorna lo stato della connessione
     *
     * @return bool
     */
    public function getStatus() {
        return $this->status;
    }

    /**
     * Ritorna l'ultima risposta del server 4Dem
     *
     * @return mixed
     */
    public function getResponse()
    {
        return $this->response;
    }
}